# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package Session;

use strict;
use warnings FATAL => 'all';
use diagnostics;

require "http.pl";
require "db.pl";
require "dbops.pl";
require "common.pl";
require "op_chan.pl";


my $cached_sid;
my %cached_session_data;

my $cached_chan_id;
my $cached_chan_name;


# r: (chan_id, chan_name)
sub guess_channel()
{
	(defined($cached_chan_id)) &&
			return ($cached_chan_id, $cached_chan_name);

	my $chan_raw = &HTTP::get("chan");
	$chan_raw = &HTTP::post("chan") if (! defined($chan_raw));

	(defined($chan_raw)) || return;

	my $chan_id = &Chan::get_id_smart($chan_raw);
	(defined($chan_id)) || return;

	my $chan_name = $chan_raw;
	if ($chan_id eq $chan_raw) {
		$chan_name = &Chan::get_name($chan_id);
	}
	$cached_chan_id = $chan_id;
	$cached_chan_name = $chan_name;

	return ($chan_id, $chan_name);
}


# 0: sid
# 1: chan id
sub save_channel($$)
{
	my ($sid, $chan) = @_;

	(defined($sid)) || return;
	($chan) = &guess_channel() if (! defined($chan));
	(! defined($chan) || $chan == -1) && return;

	my %info = &get_cookie_data($sid);
	($info{'chan'} == $chan) && return;

	my $query = "UPDATE cookies SET chan = $chan WHERE sid = '$sid'";
	&DB::update($query);
}


# r: sid
sub get_session($)
{
	(defined($cached_sid)) && return $cached_sid;

	my $sid;
	foreach my $line (&HTTP::get_cookies()) {
		my ($key, $val) = $line =~ /(.*?)=(.*)/;
		($key ne "sid") && next;
		$sid = $val;
		last;
	}

	if (! defined($sid)) {
		$sid = &HTTP::get("sid");
	}

	if (! defined($sid)) {
		return;
	}

	my ($query, $result);
	my $q_age = "EXTRACT('epoch' from now() - set)";
	$query = "SELECT ip, visits, age, $q_age as age_now FROM cookies " .
			"WHERE sid = '$sid'";
	$result = &DB::query($query);

	($result->ntuples == 0) && return;

	my ($ip, $visits, $age, $age_now) = $result->fetchrow;
	my $new_ip = &HTTP::get_ip();

	($age_now >= $age / 2) && &refresh_cookie($sid);

	$visits++;
	$query = "UPDATE cookies SET visits = $visits";
	$query .= ", ip = '$new_ip'" if (defined($new_ip) && $ip ne $new_ip);
	$query .= " WHERE sid = '$sid'";
	&DB::update($query);

	$cached_sid = $sid;

	return $sid;
}


# 0: sid
# r: (latest url id, age, urls, set_date, reset)
sub get_settings($)
{
	my ($sid) = @_;
	my ($query, $result);

	$query = "SELECT latest, age, urls, set, reset " .
			"FROM cookies WHERE sid = '$sid'";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($latest, $age, $urls, $set, $reset) = $result->fetchrow;
	$reset = ($reset eq "t") ? 1 : 0;

	return ($latest, $age, $urls, $set, $reset);
}


# 0: sid
# r: hash (age, urls, latest, reset, set, ip, visits, chan)
sub get_cookie_data($@)
{
	my $sid = shift;
	my @settings = @_;

	(defined($sid)) || return;
	(defined($cached_session_data{'set'}) && $sid eq $cached_sid) &&
			return %cached_session_data;

	my ($query, $result);

	$query = "SELECT age, urls, latest, reset, set, ip, visits, chan " .
			"FROM cookies WHERE sid = '$sid'";
	$result = &DB::query($query);
	($result->ntuples == 0) && return;

	my ($age, $urls, $latest, $reset, $set, $ip, $visits, $chan) =
			$result->fetchrow;
	my $t_reset = ($reset eq "TRUE") ? 1 : 0;
	my %ret = (
		'age' => $age,
		'urls' => $urls,
		'latest' => $latest,
		'reset' => $t_reset,
		'set' => $set,
		'ip' => $ip,
		'visits' => $visits,
		'chan' => $chan
		);

	%cached_session_data = %ret;
	
	return %ret;
}


# 0: sid
# 1: age
# 2: url count
# 3: reset
sub set_settings($$$$)
{
	my ($sid, $age, $count, $reset) = @_;

	my ($old_url_id, $old_age, $old_count, $old_date, $old_reset) =
			&Session::get_settings($sid);

	my $update = "";
	$update .= ", age = $age" if ($age != $old_age);
	$update .= ", urls = $count" if ($count != $old_count);
	if ($reset != $old_reset) {
		$update .= ", reset = " . ($reset == 1 ? "TRUE" : "FALSE");
	}
	$update =~ s/..//;

	if (length($update)) {
		my $query = "UPDATE cookies SET $update WHERE sid = '$sid'";
		&DB::update($query);
	}
}


# 0: sid
# r: chan ...
sub get_ignore_chans($)
{
	my ($sid) = @_;
	my ($query, $result);
	my @ignore;

	$query = "SELECT chan FROM chan_ignore WHERE sid = '$sid'";

	$result = &DB::query($query);
	($result->ntuples == 0) && return ();

	while (my ($chan) = $result->fetchrow) {
		push(@ignore, $chan);
	}

	return @ignore;
}


sub get_latest_url_id()
{
	my ($query, $result);

	$query = "SELECT id FROM urls ORDER BY id DESC LIMIT 1";
	$result = &DB::query($query);

	my ($latest) = $result->fetchrow;

	return $latest;
}


# 0: sid
# 1: latest
sub set_latest($$)
{
	my ($sid, $id) = @_;
	my $query;

	$query = "UPDATE cookies SET latest = $id WHERE sid = '$sid'";
	&DB::update($query);
}


# 0: sid
sub new_cookie($)
{
	my ($sid) = @_;
	my $query;

	my $latest = &Common::max(0, &get_latest_url_id() - 100);
	my $ip = &HTTP::get_ip();

	$query = "INSERT INTO cookies (sid, latest, ip) " .
			"VALUES ('$sid', $latest, '$ip')";
	&DB::update($query);
}


# 0: sid
# 1: return url
sub check_sid($)
{
	my ($sid, $ret) = @_;
	(defined($sid)) && return;
	(defined(&get_cookie_data($sid))) && return;

	&HTTP::redirect($ret);
	&HTTP::print_header();
	exit(0);
}



sub create_sid()
{
	my $try = 32;
	my $sid;
	my $used;

	for (1..16) { # try 16 sids
		$try--;
		my ($query, $result);
		my $a = rand() * (2 ** 24);
		my $b = rand() * (2 ** 24);
		my $c = rand() * (2 ** 24);
		$sid = sprintf "%06x%06x%06x", $a, $b, $c;
		$query = "SELECT sid FROM cookies WHERE sid = '$sid'";
		$result = &DB::query($query);
		if ($result->ntuples == 0) {
			&new_cookie($sid);
			return $sid;
		}
	}
	return;
}


# 0: chan
# 1: return url
sub print_links()
{
	my ($chan_id, $ret) = @_;
	my $sid = &get_session();
	
	print "<div id=\"links\">\n";
	print "\t<ul>\n";

	# Channel
	#
	my $chan_name;
	if (defined($sid) && ! defined($chan_id)) {
		my %settings = &get_cookie_data($sid);
		$chan_id = $settings{'chan'};
	}
	if (! defined($chan_id)) {
		($chan_id, $chan_name) = &guess_channel();
	}

	if (defined($chan_id) && $chan_id != -1) {
		my $chan_name = &Chan::get_name($chan_id);
		printf "\t\t<li>%s</li>\n", &HTML::htmlize($chan_name);
		print "\t\t<ul>\n";
		printf "\t\t\t<li><a href=\"%s\">%s</a></li>\n",
				&URL::chan($chan_id),
				&Lang::get("session-summary");
		printf "\t\t\t<li><a href=\"%s\">%s</a></li>\n",
				&URL::chanstat($chan_id),
				&Lang::get("session-stats");
		print "\t\t</ul>\n";
	}

	# General
	#
	printf "\t\t<li>%s</li>\n", &Lang::get("session-general");
	print "\t\t<ul>\n";
	printf "\t\t\t<li><a href=\"%s\">%s</a></li>\n",
			&URL::stat($chan_id),
			&Lang::get("session-stats");
	print "\t\t</ul>\n";

	# Session
	#
	printf "\t\t<li>%s</li>\n", &Lang::get("session-session");
	print "\t\t<ul>\n";
	if (defined($sid)) {
		printf "\t\t\t<li><a href=\"%s\">%s</a></li>\n",
				&URL::latest(),
				&Lang::get("session-latest");
		printf "\t\t\t<li><a href=\"%s\">%s</a></li>\n",
				&URL::settings(),
				&Lang::get("session-settings");
	} else {
		printf "\t\t\t<li><a href=\"%s\">%s</a></li>\n",
				&URL::session(1, $ret),
				&Lang::get("session-create");
	}
	print "\t\t</ul>\n";

	print "\t</ul>\n";
	print "</div>\n";
}


# 0: sid
sub refresh_cookie($)
{
	my ($sid) = @_;
	my %info = &get_cookie_data($sid);

	&HTTP::set_cookie("sid", $sid, $info{'age'});

	my $query .= "UPDATE cookies SET set = now() WHERE sid = '$sid'";
	&DB::update($query);
}


return 1;
