# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


package URLList;

use strict;
use warnings FATAL => 'all';
use diagnostics;

require "html.pl";
require "db.pl";
require "dbops.pl";
require "url.pl";
require "common.pl";
require "lang.pl";


# ...: column ids
sub print_columns(@)
{
	my @cols = @_;

	my %colhash;
	$colhash{$_} = 1 foreach (@cols);

	my $col_id;
	if (defined($colhash{'id'})) {
		$col_id = "\t\t<th></th>\n";
	}

	my $col_address;
	if (defined($colhash{'url'})) {
		my $l = &Lang::get("col-address");
		$col_address = "\t\t<th>$l</th>\n";
	}

	my $col_pasted;
	if (defined($colhash{'date'}) || defined($colhash{'nick'})
			|| defined($colhash{'chan'})) {
		my $l = &Lang::get("col-first-pasted");
		my $width = 0;
		$width++ if (defined($colhash{'date'}));
		$width++ if (defined($colhash{'nick'}));
		$width++ if (defined($colhash{'chan'}));
		$col_pasted = "\t\t<th colspan=\"$width\">$l</th>\n";
	}

	my $col_count;
	if (defined($colhash{'count'})) {
		my $l = &Lang::get("col-times-pasted");
		$col_count = "\t\t<th>$l</th>\n";
	}

	print "<table class=\"urls\">\n";
	print "\t<tr>\n";
	foreach (@cols) {
		print $col_id if ($_ eq "id");
		print $col_address if ($_ eq "url");
		if (defined($col_pasted) && ($_ eq "nick" || $_ eq "chan"
					|| $_ eq "date")) {
			print $col_pasted;
			undef $col_pasted;
		}
		print $col_count if ($_ eq "count");
	}
	print "\t</tr>\n";
}


# 0: hash
# 1: zebra
# 2: cols
sub print_line(\%$@)
{
	my %t = %{(shift)};
	my $zebra = shift;
	my @cols = @_;
	
	my $class = $zebra == 1 ? " class=\"zebra\"" : "";
	print "\t<tr$class>\n";
	
	foreach my $col (@cols) {
		if ($col eq "id") {
			printf "\t\t<td>%d</td>\n", $t{'url_id'};
		} elsif ($col eq "url") {
			my $t = $t{'url_url'};
			my $a = &URL::amp_url($t);
			my $lab = &URL::amp_url(&HTML::truncate_url($t));
			printf "\t\t<td><a href=\"%s\">%s</a></td>\n",
					$a, $lab;
		} elsif ($col eq "date") {
			printf "\t\t<td>%s</td>\n", $t{'url_date'};
		} elsif ($col eq "nick") {
			my $a = &URL::user($t{'nick_id'});
			my $lab = &HTML::htmlize($t{'nick_name'});
			printf "\t\t<td><a href=\"%s\">%s</a></td>\n",
					$a, $lab;
		} elsif ($col eq "chan") {
			my $a = &URL::chan($t{'chan_id'});
			my $lab = &HTML::htmlize($t{'chan_name'});
			printf "\t\t<td><a href=\"%s\">%s</a></td>\n",
					$a, $lab;
		} elsif ($col eq "count") {
			printf "\t\t<td class=\"center\">%d</td>\n",
					$t{'url_count'};
		} else {
			printf "\t\t<td>unknown</td>\n";
		}
	}
}


# 0: array of hash
# ...: columns
#	id
#	url
#	nick
#	chan
#	date
#	count
# r: image urls
sub print(\@@)
{
	my @rows = @{(shift)};
	my @cols = @_;
	my @images;

	&print_columns(@cols);

	my $i = 0;
	foreach (@rows) {
		my %t = %{$_};
		&print_line(\%t, $i % 2, @cols);
		$i++;
	}
	print <<EOF;
</table>
EOF

	return @images;
}


return 1;
